from datetime import datetime
from typing import Optional

from cpl_core.application import StartupABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import (ServiceCollectionABC,
                                           ServiceProviderABC)
from cpl_core.environment import ApplicationEnvironment
from cpl_core.logging import LoggerABC
from gismo_core.abc.bot_service_abc import BotServiceABC
from gismo_core.abc.message_service_abc import MessageServiceABC
from gismo_core.service.bot_service import BotService
from gismo_core.service.message_service import MessageService
from modules.boot_log.boot_log import BootLog
from modules_core.abc.module_abc import ModuleABC
from modules_core.abc.module_service_abc import ModuleServiceABC
from modules_core.service.module_service import ModuleService


class Startup(StartupABC):

    def __init__(self):
        StartupABC.__init__(self)
        self._start_time = datetime.now()
        
        self._config: Optional[ConfigurationABC] = None

    async def configure_configuration(self, configuration: ConfigurationABC, environment: ApplicationEnvironment) -> ConfigurationABC:
        configuration.add_environment_variables('GISMO_')
        
        configuration.add_json_file(f'appsettings.json', optional=False)
        configuration.add_json_file(f'appsettings.{environment.environment_name}.json', optional=True)
        configuration.add_json_file(f'appsettings.{environment.host_name}.json', optional=True)
        
        configuration.add_configuration('Startup_StartTime', self._start_time)
        
        self._config = configuration
        return configuration

    async def configure_services(self, services: ServiceCollectionABC, environment: ApplicationEnvironment) -> ServiceProviderABC:
        services.add_logging()
        
        services.add_singleton(ModuleServiceABC, ModuleService)
        services.add_singleton(BotServiceABC, BotService)
        services.add_transient(MessageServiceABC, MessageService)
        
        services.add_transient(ModuleABC, BootLog)
        
        provider: ServiceProviderABC = services.build_service_provider()
        
        startup_init_time = round((datetime.now() - self._config.get_configuration('Startup_StartTime')).total_seconds(), 2)
        self._config.add_configuration('Startup_InitTime', startup_init_time)
        logger: LoggerABC = provider.get_service(LoggerABC)
        logger.debug(__name__, f'Startup Init time:  {startup_init_time}s')
        
        return provider
